#include <string.h>
#include <openssl/ssl.h>
#include <openssl/bio.h>
#include <openssl/err.h>

#define HOST "www.random.org"

int main(int arc, char **argv)
{
    SSL_load_error_strings();
    ERR_load_crypto_strings();

    OpenSSL_add_all_algorithms();
    SSL_library_init();
    
    SSL_CTX *ctx = SSL_CTX_new(SSLv23_client_method());
    if (ctx == NULL) {
        printf("errored; unable to load context.\n");
        ERR_print_errors_fp(stderr);
        return -3;
    }

    BIO *bio = BIO_new_ssl_connect(ctx);

    SSL *ssl;
    BIO_get_ssl(bio, &ssl);
    SSL_set_mode(ssl, SSL_MODE_AUTO_RETRY);

    BIO_set_conn_hostname(bio, HOST":https");
    
    if (BIO_do_connect(bio) <= 0) {
        BIO_free_all(bio);
        printf("errored; unable to connect.\n");
        ERR_print_errors_fp(stderr);
        return -2;
    }

    const char *request = "GET /integers/?num=1&min=1&max=100000000&col=5&"
    "base=10&format=plain&rnd=new HTTP/1.0\nHost: "HOST"\n\n";

    if (BIO_puts(bio, request) <= 0) {
        BIO_free_all(bio);
        printf("errored; unable to write.\n");
        ERR_print_errors_fp(stderr);
        return -1;
    }

    char tmpbuf[1024+1];
    char *begin_ptr;
    char *end_ptr;
    char response[512];

    for (;;) {
        int len = BIO_read(bio, tmpbuf, 1024);
        if (len == 0) {
            break;
        }
        else if (len < 0) {
            if (!BIO_should_retry(bio)) {
                printf("errored; read failed.\n");
                ERR_print_errors_fp(stderr);
                break;
            }
        }
        else {
            tmpbuf[len] = 0;
            begin_ptr = strstr(tmpbuf, "\r\n\r\n");
            if (begin_ptr != NULL)
            {
                end_ptr = strchr(begin_ptr, '\t');
                if (end_ptr != NULL) {
                    end_ptr[0] = '\n';
                    printf("Please enter secret: ");
                    fgets(response, sizeof(response), stdin);
                    if (strcmp(response, begin_ptr+4) == 0) {
                        printf("You Win.\n");
                    } else {
                        printf("You Lose.\n");
                    }
                    break;
                }
            }
        }
    }

    BIO_free_all(bio);

    return 0;
}
